/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.io.*
import java.lang.reflect.*
import java.util.*
import java.util.regex.*

import org.codehaus.plexus.util.*

try
{
    println "Invoking class from project's test class path"
    println org.MyUtils.getNothing()

    println "Trying to access method which is unavaible in old plexus-utils"
    try
    {
        Class[] types = [ File.class, File.class ];
        println FileUtils.class.getMethod( "copyFileIfModified", types )
        println "FAILED!"
        return false
    }
    catch( NoSuchMethodException e )
    {
        // expected
        e.printStackTrace()
    }
      
}
catch( Throwable t )
{
    t.printStackTrace()
    return false
}
